@@ -150,6 +150,10 @@ static struct tegra_gpio_table gpio_table[] = {
 	{ .gpio = TEGRA_WIFI_LED,	.enable = true },
 	{ .gpio = TEGRA_NVEC_REQ,	.enable = true },
 	{ .gpio = TEGRA_HP_DET,		.enable = true },
+	{ .gpio = TEGRA_BACKLIGHT,	.enable = true },
+	{ .gpio = TEGRA_BACKLIGHT_VDD,	.enable = true },
+	{ .gpio = TEGRA_LVDS_SHUTDOWN,	.enable = true },
+	{ .gpio = TEGRA_EN_VDD_PNL,	.enable = true },
 	{ .gpio = TEGRA_HDMI_HPD,	.enable = true },
 };
 
