@@ -31,6 +31,7 @@
 #include <linux/io.h>
 #include <linux/rfkill-gpio.h>
 #include <linux/mfd/core.h>
+#include <linux/platform_data/tegra_usb.h>
 
 #include <asm/mach-types.h>
 #include <asm/mach/arch.h>
@@ -154,9 +155,9 @@ static struct plat_serial8250_port debug_uart_platform_data[] = {
 		.uartclk	= 216000000,
 	}, {
 		/* serial port on mini-pcie */
-		.membase	= IO_ADDRESS(TEGRA_UARTD_BASE),
-		.mapbase	= TEGRA_UARTD_BASE,
-		.irq		= INT_UARTD,
+		.membase	= IO_ADDRESS(TEGRA_UARTC_BASE),
+		.mapbase	= TEGRA_UARTC_BASE,
+		.irq		= INT_UARTC,
 		.flags		= UPF_BOOT_AUTOCONF | UPF_FIXED_TYPE,
 		.type		= PORT_TEGRA,
 		.iotype		= UPIO_MEM,
@@ -294,7 +295,7 @@ static struct tegra_alc5632_audio_platform_data audio_pdata = {
 };
 
 static struct platform_device audio_device = {
-	.name	= "tegra-snd-alc5632",
+	.name	= "tegra-alc5632",
 	.id	= 0,
 	.dev	= {
 		.platform_data = &audio_pdata,
@@ -325,6 +326,7 @@ static struct platform_device *paz00_devices[] __initdata = {
 	&audio_device,
 	&tegra_avp_device,
 	&tegra_aes_device,
+	&tegra_wdt_device,
 };
 
 static struct tegra_i2c_platform_data paz00_i2c1_platform_data = {
@@ -384,6 +386,11 @@ static void __init paz00_i2c_init(void)
 
 static void paz00_usb_init(void)
 {
+	struct tegra_ehci_platform_data *pdata;
+
+	pdata = tegra_ehci3_device.dev.platform_data;
+	pdata->power_down_on_bus_suspend = 0;
+
 	platform_device_register(&tegra_ehci2_device);
 	platform_device_register(&tegra_ehci3_device);
 }
@@ -399,7 +406,7 @@ static void __init tegra_paz00_fixup(struct machine_desc *desc,
 static __initdata struct tegra_clk_init_table paz00_clk_init_table[] = {
 	/* name		parent		rate		enabled */
 	{ "uarta",	"pll_p",	216000000,	true  },
-	{ "uartd",	"pll_p",	216000000,	true  },
+	{ "uartc",	"pll_p",	216000000,	true  },
 
 	{ "pll_a",	"pll_p_out1",	56448000,	true  },
 	{ "pll_a_out0",	"pll_a",	11289600,	true  },
@@ -408,8 +415,12 @@ static __initdata struct tegra_clk_init_table paz00_clk_init_table[] = {
 	{ "usb2",	"clk_m",	12000000,	false },
 	{ "usb3",	"clk_m",	12000000,	false },
 
+	/* needed for bootloaders without display support */
+	{ "disp1",	"pll_p",	216000000,	true  },
+
 	/* these are used for audio */
 	{ "cdev1",	NULL,		0,		true  },
+	{ "audio",	"pll_a_out0",	11289600,	false },
 	{ "audio_2x",	"audio",	22579200,	false },
 	{ "i2s1",	"pll_a_out0",	11289600,	false },
 
