@@ -264,7 +264,7 @@ int drm_fb_helper_panic(struct notifier_block *n, unsigned long ununsed,
 	 * if the kernel should reboot before panic messages can be seen.
 	 */
 	if (panic_timeout >= 0) {
-		DRM_ERROR("panic occurred, switching back to text console\n");
+		printk(KERN_ERR "panic occurred, switching back to text console\n");
 		drm_fb_helper_force_kernel_mode();
 	}
 	return 0;
