@@ -97,7 +97,7 @@ static int chromeos_access_nvram_block(struct page *page,
 	bio->bi_io_vec[0].bv_offset	= 0;
 
 	/* submit bio */
-	rq = REQ_SYNC | REQ_SOFTBARRIER | REQ_UNPLUG | REQ_NOIDLE;
+	rq = REQ_SYNC | REQ_SOFTBARRIER | REQ_NOIDLE;
 	if (!is_read)
 		rq |= REQ_WRITE;
 
