@@ -1010,9 +1010,8 @@ void fb_edid_add_monspecs(unsigned char *edid, struct fb_monspecs *specs)
 			}
 		else if (type == 0x3) {
 			if (len >= 3) {
-				u32 ieee_reg = edid[pos+1] |
-					(edid[pos+2] << 8) |
-					(edid[pos+3] << 16);
+				u32 ieee_reg = edid[pos] | (edid[pos+1] << 8) |
+					(edid[pos+2] << 16);
 				if (ieee_reg == 0x000c03)
 					specs->misc |= FB_MISC_HDMI;
 			}
