@@ -78,6 +78,8 @@ struct nvmap_device {
 	struct nvmap_carveout_node *heaps;
 	int nr_carveouts;
 	struct nvmap_share iovmm_master;
+	struct list_head clients;
+	spinlock_t	clients_lock;
 };
 
 struct nvmap_device *nvmap_dev;
@@ -627,6 +629,9 @@ struct nvmap_client *nvmap_create_client(struct nvmap_device *dev,
 	spin_lock_init(&client->ref_lock);
 	atomic_set(&client->count, 1);
 
+	spin_lock(&dev->clients_lock);
+	list_add(&client->list, &dev->clients);
+	spin_unlock(&dev->clients_lock);
 	return client;
 }
 EXPORT_SYMBOL_GPL(nvmap_create_client);
@@ -677,6 +682,9 @@ static void destroy_client(struct nvmap_client *client)
 	if (client->task)
 		put_task_struct(client->task);
 
+	spin_lock(&client->dev->clients_lock);
+	list_del(&client->list);
+	spin_unlock(&client->dev->clients_lock);
 	kfree(client);
 }
 
@@ -929,11 +937,11 @@ static void client_stringify(struct nvmap_client *client, struct seq_file *s)
 {
 	char task_comm[TASK_COMM_LEN];
 	if (!client->task) {
-		seq_printf(s, "%8s %16s %8u", client->name, "kernel", 0);
+		seq_printf(s, "%-18s %18s %8u", client->name, "kernel", 0);
 		return;
 	}
 	get_task_comm(task_comm, client->task);
-	seq_printf(s, "%8s %16s %8u", client->name, task_comm,
+	seq_printf(s, "%-18s %18s %8u", client->name, task_comm,
 		   client->task->pid);
 }
 
@@ -941,39 +949,63 @@ static void allocations_stringify(struct nvmap_client *client,
 				  struct seq_file *s)
 {
 	struct rb_node *n = rb_first(&client->handle_refs);
-	unsigned long long total = 0;
+	const unsigned long base = 0;
 
 	for (; n != NULL; n = rb_next(n)) {
 		struct nvmap_handle_ref *ref =
 			rb_entry(n, struct nvmap_handle_ref, node);
 		struct nvmap_handle *handle = ref->handle;
-		if (handle->alloc && !handle->heap_pgalloc) {
-			seq_printf(s, " %8u@%8lx ", handle->size,
-				   handle->carveout->base);
-			total += handle->size;
-		}
+		if (!handle->alloc)
+			continue;
+
+		if (!handle->heap_pgalloc)
+			seq_printf(s, "%-18s %-18s %8lx %10u\n", "", "",
+					(unsigned long)(handle->carveout->base),
+					handle->size);
+		else
+			seq_printf(s, "%-18s %-18s %8lx %10u\n", "", "",
+					base, handle->size);
 	}
-	seq_printf(s, " total: %llu\n", total);
 }
 
-static int nvmap_debug_allocations_show(struct seq_file *s, void *unused)
+#define NVMAP_DEBUGFS_MODE_ALLOCATIONS	0
+#define NVMAP_DEBUGFS_MODE_CLIENTS	1
+static int nvmap_debug_show(struct seq_file *s, int mode)
 {
 	struct nvmap_carveout_node *node = s->private;
 	struct nvmap_carveout_commit *commit;
 	unsigned long flags;
+	unsigned int total = 0;
 
 	spin_lock_irqsave(&node->clients_lock, flags);
+	seq_printf(s, "%-18s %18s %8s %10s\n", "CLIENT", "PROCESS", "PID",
+			"SIZE");
+	if (mode == NVMAP_DEBUGFS_MODE_ALLOCATIONS)
+		seq_printf(s, "%-18s %18s %8s %10s\n", "", "",
+						"BASE", "SIZE");
+
 	list_for_each_entry(commit, &node->clients, list) {
 		struct nvmap_client *client =
 			get_client_from_carveout_commit(node, commit);
 		client_stringify(client, s);
-		allocations_stringify(client, s);
+		seq_printf(s, " %10u\n", commit->commit);
+		if (mode == NVMAP_DEBUGFS_MODE_ALLOCATIONS) {
+			allocations_stringify(client, s);
+			seq_printf(s, "\n");
+		}
+		total += commit->commit;
 	}
+	seq_printf(s, "%-18s %-18s %8u %10u\n", "total", "", 0, total);
 	spin_unlock_irqrestore(&node->clients_lock, flags);
 
 	return 0;
 }
 
+static int nvmap_debug_allocations_show(struct seq_file *s, void *unused)
+{
+	return nvmap_debug_show(s, NVMAP_DEBUGFS_MODE_ALLOCATIONS);
+}
+
 static int nvmap_debug_allocations_open(struct inode *inode, struct file *file)
 {
 	return single_open(file, nvmap_debug_allocations_show,
@@ -989,20 +1021,7 @@ static struct file_operations debug_allocations_fops = {
 
 static int nvmap_debug_clients_show(struct seq_file *s, void *unused)
 {
-	struct nvmap_carveout_node *node = s->private;
-	struct nvmap_carveout_commit *commit;
-	unsigned long flags;
-
-	spin_lock_irqsave(&node->clients_lock, flags);
-	list_for_each_entry(commit, &node->clients, list) {
-		struct nvmap_client *client =
-			get_client_from_carveout_commit(node, commit);
-		client_stringify(client, s);
-		seq_printf(s, " %8u\n", commit->commit);
-	}
-	spin_unlock_irqrestore(&node->clients_lock, flags);
-
-	return 0;
+	return nvmap_debug_show(s, NVMAP_DEBUGFS_MODE_CLIENTS);
 }
 
 static int nvmap_debug_clients_open(struct inode *inode, struct file *file)
@@ -1017,6 +1036,74 @@ static struct file_operations debug_clients_fops = {
 	.release = single_release,
 };
 
+static int nvmap_debug_iovmm_show(struct seq_file *s, int mode)
+{
+	unsigned long flags;
+	unsigned int total = 0;
+	struct nvmap_client *client;
+	struct nvmap_device *dev = s->private;
+
+	spin_lock_irqsave(&dev->clients_lock, flags);
+	seq_printf(s, "%-18s %18s %8s %10s\n", "CLIENT", "PROCESS", "PID",
+		"SIZE");
+	if (mode == NVMAP_DEBUGFS_MODE_ALLOCATIONS)
+		seq_printf(s, "%-18s %18s %8s %10s\n", "", "",
+						"BASE", "SIZE");
+
+	list_for_each_entry(client, &dev->clients, list) {
+		int iovm_commit = atomic_read(&client->iovm_commit);
+		client_stringify(client, s);
+		seq_printf(s, " %10u\n", iovm_commit);
+		if (mode == NVMAP_DEBUGFS_MODE_ALLOCATIONS) {
+			allocations_stringify(client, s);
+			seq_printf(s, "\n");
+		}
+		total += iovm_commit;
+	}
+	seq_printf(s, "%-18s %18s %8u %10u\n", "total", "", 0, total);
+	spin_unlock_irqrestore(&dev->clients_lock, flags);
+
+	return 0;
+}
+
+static int nvmap_debug_iovmm_clients_show(struct seq_file *s, void *unused)
+{
+	return nvmap_debug_iovmm_show(s, NVMAP_DEBUGFS_MODE_CLIENTS);
+}
+
+static int nvmap_debug_iovmm_clients_open(struct inode *inode,
+					    struct file *file)
+{
+	return single_open(file, nvmap_debug_iovmm_clients_show,
+			    inode->i_private);
+}
+
+static const struct file_operations debug_iovmm_clients_fops = {
+	.open = nvmap_debug_iovmm_clients_open,
+	.read = seq_read,
+	.llseek = seq_lseek,
+	.release = single_release,
+};
+
+static int nvmap_debug_iovmm_allocations_show(struct seq_file *s, void *unused)
+{
+	return nvmap_debug_iovmm_show(s, NVMAP_DEBUGFS_MODE_ALLOCATIONS);
+}
+
+static int nvmap_debug_iovmm_allocations_open(struct inode *inode,
+						struct file *file)
+{
+	return single_open(file, nvmap_debug_iovmm_allocations_show,
+			    inode->i_private);
+}
+
+static const struct file_operations debug_iovmm_allocations_fops = {
+	.open = nvmap_debug_iovmm_allocations_open,
+	.read = seq_read,
+	.llseek = seq_lseek,
+	.release = single_release,
+};
+
 static int nvmap_probe(struct platform_device *pdev)
 {
 	struct nvmap_platform_data *plat = pdev->dev.platform_data;
@@ -1078,6 +1165,8 @@ static int nvmap_probe(struct platform_device *pdev)
 
 	spin_lock_init(&dev->ptelock);
 	spin_lock_init(&dev->handle_lock);
+	INIT_LIST_HEAD(&dev->clients);
+	spin_lock_init(&dev->clients_lock);
 
 	for (i = 0; i < NVMAP_NUM_PTES; i++) {
 		unsigned long addr;
@@ -1161,13 +1250,23 @@ static int nvmap_probe(struct platform_device *pdev)
 			struct dentry *heap_root =
 				debugfs_create_dir(co->name, nvmap_debug_root);
 			if (!IS_ERR_OR_NULL(heap_root)) {
-				debugfs_create_file("clients", 0664, heap_root,
+				debugfs_create_file("clients", 0444, heap_root,
 				    node, &debug_clients_fops);
-				debugfs_create_file("allocations", 0664,
+				debugfs_create_file("allocations", 0444,
 				    heap_root, node, &debug_allocations_fops);
 			}
 		}
 	}
+	if (!IS_ERR_OR_NULL(nvmap_debug_root)) {
+		struct dentry *iovmm_root =
+			debugfs_create_dir("iovmm", nvmap_debug_root);
+		if (!IS_ERR_OR_NULL(iovmm_root)) {
+			debugfs_create_file("clients", 0444, iovmm_root,
+				dev, &debug_iovmm_clients_fops);
+			debugfs_create_file("allocations", 0444, iovmm_root,
+				dev, &debug_iovmm_allocations_fops);
+		}
+	}
 
 	platform_set_drvdata(pdev, dev);
 	nvmap_dev = dev;
